@@ -62,6 +62,7 @@ import org.eclipse.cdt.core.dom.ast.IVariable;
 import org.eclipse.cdt.core.dom.ast.c.ICASTTypeIdInitializerExpression;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPASTLinkageSpecification;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPASTNamespaceDefinition;
+import org.eclipse.cdt.core.dom.ast.gnu.cpp.GPPLanguage;
 import org.eclipse.cdt.core.dom.parser.IScannerExtensionConfiguration;
 import org.eclipse.cdt.core.dom.parser.c.ANSICParserExtensionConfiguration;
 import org.eclipse.cdt.core.dom.parser.c.GCCParserExtensionConfiguration;
@@ -114,8 +115,8 @@ public class AST2TestBase extends BaseTestCase {
 
 	private static Map<String, String> getGnuMap() {
 		Map<String, String> map= new HashMap<String, String>();
-		map.put("__GNUC__", "4");
-		map.put("__GNUC_MINOR__", "7");
+		map.put("__GNUC__", Integer.toString(GPPLanguage.GNU_LATEST_VERSION_MAJOR));
+		map.put("__GNUC_MINOR__", Integer.toString(GPPLanguage.GNU_LATEST_VERSION_MINOR));
 		map.put("__SIZEOF_SHORT__", "2");
 		map.put("__SIZEOF_INT__", "4");
 		map.put("__SIZEOF_LONG__", "8");
@@ -156,11 +157,11 @@ public class AST2TestBase extends BaseTestCase {
 
     protected IASTTranslationUnit parse(String code, ParserLanguage lang, boolean useGNUExtensions,
     		boolean expectNoProblems) throws ParserException {
-    	return parse(code, lang, useGNUExtensions, expectNoProblems, false);
+    	return parse(code, lang, useGNUExtensions, expectNoProblems, Integer.MAX_VALUE);
     }
 
     protected IASTTranslationUnit parse(String code, ParserLanguage lang, boolean useGNUExtensions,
-    		boolean expectNoProblems, boolean skipTrivialInitializers) throws ParserException {
+    		boolean expectNoProblems, int limitTrivialInitializers) throws ParserException {
 		IScanner scanner = createScanner(FileContent.create(TEST_CODE, code.toCharArray()), lang, ParserMode.COMPLETE_PARSE,
         		createScannerInfo(useGNUExtensions));
         configureScanner(scanner);
@@ -184,8 +185,7 @@ public class AST2TestBase extends BaseTestCase {
 
             parser = new GNUCSourceParser(scanner, ParserMode.COMPLETE_PARSE, NULL_LOG, config, null);
         }
-        if (skipTrivialInitializers)
-        	parser.setSkipTrivialExpressionsInAggregateInitializers(true);
+        parser.setMaximumTrivialExpressionsInAggregateInitializers(limitTrivialInitializers);
 
         IASTTranslationUnit tu = parser.parse();
         assertTrue(tu.isFrozen());
@@ -749,12 +749,12 @@ public class AST2TestBase extends BaseTestCase {
 	}
 
 	final protected IASTTranslationUnit parseAndCheckBindings(String code, ParserLanguage lang, boolean useGnuExtensions) throws Exception {
-		return parseAndCheckBindings(code, lang, useGnuExtensions, false);
+		return parseAndCheckBindings(code, lang, useGnuExtensions, Integer.MAX_VALUE);
 	}
 
 	final protected IASTTranslationUnit parseAndCheckBindings(String code, ParserLanguage lang, boolean useGnuExtensions,
-			boolean skipTrivialInitializers) throws Exception {
-		IASTTranslationUnit tu = parse(code, lang, useGnuExtensions, true, skipTrivialInitializers);
+			int limitTrvialInitializers) throws Exception {
+		IASTTranslationUnit tu = parse(code, lang, useGnuExtensions, true, limitTrvialInitializers);
 		NameCollector col = new NameCollector();
 		tu.accept(col);
 		assertNoProblemBindings(col);
