@@ -2383,4 +2383,90 @@ public class IndexCPPTemplateResolutionTest extends IndexBindingResolutionTestBa
 	public void testAmbiguousBaseClassLookup_413406() throws Exception {
 		getProblemFromASTName("eof();", 3);
 	}
+
+	//	template <typename = void>
+	//	struct S;
+
+	//	#include "header.h"
+	//	template <>
+	//	struct S<void> {
+	//		typedef int type;
+	//	};
+	//
+	//	template <typename>
+	//	struct S {
+	//		typedef char type;
+	//	};
+	//
+	//	typedef S<>::type T;
+	public void testExplicitSpecializationOfTemplateDeclaredInHeader_401820() throws Exception {
+		IType T = getBindingFromASTName("T", 1);
+		assertEquals("int", ASTTypeUtil.getType(T));
+	}
+
+	//	template <class T, class U>
+	//	struct multipliable2
+	//	{
+	//	    friend T operator *(const U& lhs, const T& rhs);
+	//	};
+	//
+	//	template <class T>
+	//	struct multipliable1
+	//	{
+	//	    friend T operator *(const T& lhs, const T& rhs);
+	//	};
+
+	//	#include "header.h"
+	//	struct overloaded : multipliable1<overloaded> {};
+	//
+	//	int foo(overloaded);
+	//
+	//	int main()
+	//	{
+	//	    overloaded c, d;
+	//	    foo(c * d);
+	//	}
+	public void testFriendFunctionOfClassSpecialization_419301a() throws Exception {
+		checkBindings();
+	}
+
+	//	template <class T, class U>
+	//	struct multipliable2
+	//	{
+	//	    friend T operator *(const U& lhs, const T& rhs);
+	//	};
+	//
+	//	template <class T>
+	//	struct multipliable1
+	//	{
+	//	    friend T operator *(const T& lhs, const T& rhs) {}
+	//	};
+
+	//	#include "header.h"
+	//	struct overloaded : multipliable1 <overloaded> {};
+	//
+	//	int foo(overloaded);
+	//
+	//	int main() {
+	//	    overloaded c, d;
+	//	    foo(c * d);
+	//	}
+	public void testFriendFunctionOfClassSpecialization_419301b() throws Exception {
+		checkBindings();
+	}
+	
+	//	template <typename T>
+	//	constexpr T t(T) {
+	//	    return 0;
+	//	}
+	//
+	//	template <>
+	//	constexpr unsigned t<unsigned>(unsigned) {
+	//	    return 1 + 1;
+	//	}
+	
+	//	// empty source file
+	public void testSpecializationOfConstexprFunction_420995() throws Exception {
+		checkBindings();
+	}
 }
