@@ -8,8 +8,8 @@
  * Contributors:
  *     Andrew Gvozdev - Initial API and implementation
  *     James Blackburn (Broadcom Corp.)
+ *     Liviu Ionescu - bug 392416
  *******************************************************************************/
-
 package org.eclipse.cdt.core.testplugin;
 
 import java.io.BufferedReader;
@@ -27,8 +27,6 @@ import java.util.HashSet;
 import java.util.Set;
 import java.util.UUID;
 
-import junit.framework.Assert;
-
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.dom.IPDOMManager;
 import org.eclipse.cdt.core.settings.model.ICConfigurationDescription;
@@ -54,6 +52,7 @@ import org.eclipse.core.runtime.OperationCanceledException;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.jobs.Job;
+import org.junit.Assert;
 
 /**
  * This class contains utility methods for creating resources
@@ -66,7 +65,6 @@ import org.eclipse.core.runtime.jobs.Job;
  *
  * @since 6.0
  */
-
 public class ResourceHelper {
 	private final static IProgressMonitor NULL_MONITOR = new NullProgressMonitor();
 	private static final int MAX_RETRY= 5;
@@ -112,7 +110,7 @@ public class ResourceHelper {
 			prjDescription.setLocation(absoluteLocation);
 		}
 
-		if (configurationIds!=null && configurationIds.length>0) {
+		if (configurationIds != null && configurationIds.length > 0) {
 			ICProjectDescriptionManager prjDescManager = cdtCorePlugin.getProjectDescriptionManager();
 
 			project.create(NULL_MONITOR);
@@ -122,7 +120,7 @@ public class ResourceHelper {
 			ICConfigurationDescription baseConfiguration = cdtCorePlugin.getPreferenceConfiguration(TestCfgDataProvider.PROVIDER_ID);
 
 			for (String cfgId : configurationIds) {
-				icPrjDescription.createConfiguration(cfgId, cfgId+" Name", baseConfiguration);
+				icPrjDescription.createConfiguration(cfgId, cfgId + " Name", baseConfiguration);
 			}
 			prjDescManager.setProjectDescription(project, icPrjDescription);
 		}
@@ -193,7 +191,7 @@ public class ResourceHelper {
 	}
 
 	/**
-	 * Create a plain Eclipse project.
+	 * Creates a plain Eclipse project.
 	 *
 	 * @param projectName
 	 * @return  the project handle
@@ -202,10 +200,11 @@ public class ResourceHelper {
 	public static IProject createProject(String projectName) throws CoreException {
 		IWorkspaceRoot root= ResourcesPlugin.getWorkspace().getRoot();
 		IProject project= root.getProject(projectName);
-		if (!project.exists())
+		if (!project.exists()) {
 			project.create(NULL_MONITOR);
-		else
+		} else {
 			project.refreshLocal(IResource.DEPTH_INFINITE, null);
+		}
 
 		if (!project.isOpen())
 			project.open(NULL_MONITOR);
@@ -215,7 +214,7 @@ public class ResourceHelper {
 	}
 
 	/**
-	 * Delete project by name.
+	 * Deletes project by name.
 	 *
 	 * @param projectName
 	 * @throws CoreException
@@ -228,7 +227,7 @@ public class ResourceHelper {
 	}
 
 	/**
-	 * Delete given project with content.
+	 * Deletes given project with content.
 	 *
 	 * @param project
 	 * @throws CoreException
@@ -238,7 +237,7 @@ public class ResourceHelper {
 	}
 
 	/**
-	 * Delete project.
+	 * Deletes project.
 	 *
 	 * @param project
 	 * @param deleteContent  whether to delete project content
@@ -377,7 +376,7 @@ public class ResourceHelper {
 	 * @throws IOException if something goes wrong.
 	 */
 	public static IPath createTemporaryFolder() throws CoreException, IOException {
-		return ResourceHelper.createWorkspaceFolder("tmp/"+System.currentTimeMillis()+'.'+UUID.randomUUID());
+		return ResourceHelper.createWorkspaceFolder("tmp/" + System.currentTimeMillis() + '.' + UUID.randomUUID());
 	}
 
 	/**
@@ -541,13 +540,12 @@ public class ResourceHelper {
 	 * @throws CoreException...
 	 */
 	public static IResource createSymbolicLink(IProject project, String linkName, IPath realPath)
-		throws IOException, CoreException, UnsupportedOperationException {
-
+			throws IOException, CoreException, UnsupportedOperationException {
 		if (!isSymbolicLinkSupported()) {
 			throw new UnsupportedOperationException("Windows links .lnk are not supported.");
 		}
 
-		Assert.assertTrue("Path for symbolic link does not exist: [" + realPath.toOSString() +"]",
+		Assert.assertTrue("Path for symbolic link does not exist: [" + realPath.toOSString() + "]",
 				new File(realPath.toOSString()).exists());
 
 		IPath linkedPath = project.getLocation().append(linkName);
@@ -582,23 +580,23 @@ public class ResourceHelper {
 			throw new UnsupportedOperationException("Windows links .lnk are not supported.");
 		}
 
-		String command = "ln -s " + realPath.toOSString() + ' ' + linkPath.toOSString();
+		String command[] = { "ln", "-s", realPath.toOSString(), linkPath.toOSString()};
 		Process process = Runtime.getRuntime().exec(command);
 
 		// Wait for up to 2.5s...
 		for (int i = 0; i < 5; i++) {
 			try {
 				Assert.assertTrue("ln process exited with non-zero status", process.waitFor() == 0);
-				// If exitValue succeeded, then the process has exitted successfully.
+				// If exitValue succeeded, then the process has exited successfully.
 				break;
 			} catch (InterruptedException e) {
 				// Clear interrupted state, see Java bug http://bugs.sun.com/view_bug.do?bug_id=6420270
 				Thread.interrupted();
 			}
-			// wait for a 500ms before checking again
+			// Wait for a 500ms before checking again.
 			try { Thread.sleep(500); } catch (InterruptedException e) {/*don't care*/}
 		}
-		Assert.assertTrue("Symbolic link not created, command=[" + command +"]", linkPath.toFile().exists());
+		Assert.assertTrue("Symbolic link not created, command=[" + command + "]", linkPath.toFile().exists());
 	}
 
 	/**
@@ -616,8 +614,7 @@ public class ResourceHelper {
 	 * @throws CoreException...
 	 */
 	public static IResource createSymbolicLink(IProject project, String linkName, String realPath)
-		throws IOException, CoreException, UnsupportedOperationException {
-
+			throws IOException, CoreException, UnsupportedOperationException {
 		return createSymbolicLink(project, linkName, new Path(realPath));
 	}
 
@@ -701,12 +698,13 @@ public class ResourceHelper {
 
 		// Remove IResources created by this helper
 		for (IResource r : resourcesCreated) {
-			if (r.exists())
+			if (r.exists()) {
 				try {
 					r.delete(true, NULL_MONITOR);
 				} catch (CoreException e) {
 					// Ignore
 				}
+			}
 		}
 		resourcesCreated.clear();
 	}
@@ -732,13 +730,16 @@ public class ResourceHelper {
 	private static final void deleteRecursive(File f) throws IllegalArgumentException {
 		// Ensure that the file being deleted is a child of the workspace
 		// root to prevent anything nasty happening
-		if (! f.getAbsolutePath().startsWith(
-				ResourcesPlugin.getWorkspace().getRoot().getLocation().toFile().getAbsolutePath()))
+		if (!f.getAbsolutePath().startsWith(
+				ResourcesPlugin.getWorkspace().getRoot().getLocation().toFile().getAbsolutePath())) {
 			throw new IllegalArgumentException("File must exist within the workspace!");
+		}
 
-		if (f.isDirectory())
-			for (File f1 : f.listFiles())
+		if (f.isDirectory()) {
+			for (File f1 : f.listFiles()) {
 				deleteRecursive(f1);
+			}
+		}
 		f.delete();
 	}
 }
