@@ -19,7 +19,9 @@ import org.eclipse.cdt.core.parser.IScannerInfo;
  * @author jcamelon
  * @noextend This interface is not intended to be extended by clients.
  * @noimplement This interface is not intended to be implemented by clients.
+ * @deprecated Used only by other deprecated interfaces
  */
+@Deprecated
 public interface IParserConfiguration {
 	/**
 	 * @return IScannerInfo representing the build information required to parse.
