@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2012 IBM Corporation and others.
+ * Copyright (c) 2005, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
  *     Ed Swartz (Nokia)
  *     Mike Kucera (IBM) - bug #206952
  *     Sergey Prigogin (Google)
+ *     Thomas Corbat (IFS)
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.dom.parser;
 
@@ -188,7 +189,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
     protected final IBuiltinBindingsProvider builtinBindingsProvider;
 
     protected boolean functionCallCanBeLValue= false;
-	protected boolean skipTrivialExpressionsInAggregateInitializers= false;
+	protected int maximumTrivialExpressionsInAggregateInitializers= Integer.MAX_VALUE;
 
     /**
      *  Marks the beginning of the current declaration. It is important to clear the mark whenever we
@@ -232,10 +233,10 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
 
 	/**
 	 * Instructs the parser not to create ast nodes for expressions within aggregate initializers
-	 * when they do not contain names.
+	 * when they do not contain names beyond the given limit.
 	 */
-	public void setSkipTrivialExpressionsInAggregateInitializers(boolean val) {
-		skipTrivialExpressionsInAggregateInitializers= val;
+	public void setMaximumTrivialExpressionsInAggregateInitializers(int limit) {
+		maximumTrivialExpressionsInAggregateInitializers= limit;
 	}
 
     private AbstractParserLogService wrapLogService(IParserLogService logService) {
@@ -676,8 +677,10 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
 
     protected abstract void nullifyTranslationUnit();
 
-	protected IToken skipOverCompoundStatement() throws BacktrackException, EndOfFileException {
+	protected IToken skipOverCompoundStatement(boolean hasSkippedNodes) throws BacktrackException, EndOfFileException {
         // speed up the parser by skipping the body, simply look for matching brace and return
+		if (hasSkippedNodes)
+			getTranslationUnit().setHasNodesOmitted(true);
         final boolean isActive = isActiveCode();
 		final int codeBranchNesting= getCodeBranchNesting();
 
@@ -904,13 +907,13 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
     protected IASTExpression compoundStatementExpression() throws EndOfFileException, BacktrackException {
         int startingOffset = consume().getOffset(); // tLPAREN always
         IASTCompoundStatement compoundStatement = null;
-        if (mode == ParserMode.QUICK_PARSE || mode == ParserMode.STRUCTURAL_PARSE || !isActiveCode())
-            skipOverCompoundStatement();
-        else if (mode == ParserMode.COMPLETION_PARSE || mode == ParserMode.SELECTION_PARSE) {
+        if (mode == ParserMode.QUICK_PARSE || mode == ParserMode.STRUCTURAL_PARSE || !isActiveCode()) {
+            skipOverCompoundStatement(true);
+        } else if (mode == ParserMode.COMPLETION_PARSE || mode == ParserMode.SELECTION_PARSE) {
             if (scanner.isOnTopContext())
                 compoundStatement();
             else
-                skipOverCompoundStatement();
+                skipOverCompoundStatement(true);
         } else if (mode == ParserMode.COMPLETE_PARSE)
             compoundStatement = compoundStatement();
 
@@ -1329,7 +1332,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
 				} catch (OffsetLimitReachedException e) {
 					return;
 				}
-				codeBranchNesting= Math.min(getCodeBranchNesting()+1, codeBranchNesting);
+				codeBranchNesting= Math.min(getCodeBranchNesting() + 1, codeBranchNesting);
 
 				// we could be at the start of inactive code so restart the loop
 				continue;
@@ -1428,7 +1431,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
     	declarationMark= null;
         if (mode == ParserMode.QUICK_PARSE || mode == ParserMode.STRUCTURAL_PARSE || !isActiveCode()) {
             int offset = LA(1).getOffset();
-            IToken last = skipOverCompoundStatement();
+            IToken last = skipOverCompoundStatement(true);
             IASTCompoundStatement cs = nodeFactory.newCompoundStatement();
             setRange(cs, offset, last.getEndOffset());
             return cs;
@@ -1436,7 +1439,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
             if (scanner.isOnTopContext())
                 return functionBody();
             int offset = LA(1).getOffset();
-            IToken last = skipOverCompoundStatement();
+            IToken last = skipOverCompoundStatement(true);
             IASTCompoundStatement cs = nodeFactory.newCompoundStatement();
             setRange(cs, offset, last.getEndOffset());
             return cs;
@@ -1620,16 +1623,16 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
     	} catch (BacktrackException e) {
 			if (acceptEmpty) {
 				backup(dtorMark1);
-				return result.set(declspec1, null, null);
+				dtor1= null;
+			} else {
+				// try second variant, if possible
+				if (dtorMark2 == null)
+					throw e;
+	
+				backup(dtorMark2);
+				dtor2= initDeclarator(declspec2, option);
+				return result.set(declspec2, dtor2, dtorMark2);
 			}
-
-			// try second variant, if possible
-			if (dtorMark2 == null)
-				throw e;
-
-			backup(dtorMark2);
-			dtor2= initDeclarator(declspec2, option);
-			return result.set(declspec2, dtor2, dtorMark2);
 		}
 
     	// first variant was a success. If possible, try second one.
@@ -1770,7 +1773,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
     		throwBacktrack(offset, LA(1).getEndOffset() - offset);
 
     	final int compoundOffset= LA(1).getOffset();
-    	final int endOffset= skipOverCompoundStatement().getEndOffset();
+    	final int endOffset= skipOverCompoundStatement(false).getEndOffset();
     	IASTCompoundStatement cs = nodeFactory.newCompoundStatement(); //createCompoundStatement();
     	((ASTNode)cs).setOffsetAndLength(compoundOffset, endOffset - compoundOffset);
 
@@ -2413,7 +2416,7 @@ public abstract class AbstractGNUSourceCodeParser implements ISourceCodeParser {
 		while ((t = LA(1)).getType() != endType) {
 			consume();
 			IASTToken token;
-			switch (LT(1)) {
+			switch (t.getType()) {
 			case IToken.tLPAREN:
 				token = balancedTokenSeq(t.getOffset(), IToken.tRPAREN);
 				break;
