@@ -31,6 +31,7 @@ import org.eclipse.cdt.core.dom.ast.ASTTypeUtil;
 import org.eclipse.cdt.core.dom.ast.DOMException;
 import org.eclipse.cdt.core.dom.ast.IASTExpression.ValueCategory;
 import org.eclipse.cdt.core.dom.ast.IASTNode;
+import org.eclipse.cdt.core.dom.ast.IASTUnaryExpression;
 import org.eclipse.cdt.core.dom.ast.IArrayType;
 import org.eclipse.cdt.core.dom.ast.IBasicType;
 import org.eclipse.cdt.core.dom.ast.IBinding;
@@ -58,6 +59,7 @@ import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateInstance;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateNonTypeParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateTemplateParameter;
+import org.eclipse.cdt.internal.core.dom.parser.ITypeContainer;
 import org.eclipse.cdt.internal.core.dom.parser.Value;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPBasicType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPPointerToMemberType;
@@ -69,6 +71,7 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPTemplateTypeArgument;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ClassTypeHelper;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPEvaluation;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPUnknownBinding;
+import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPUnknownMember;
 
 /**
  * Algorithms for deducing template arguments in various contexts.
@@ -90,7 +93,7 @@ public class TemplateArgumentDeduction {
 			CPPTemplateParameterMap map, IASTNode point) throws DOMException {
 		final ICPPTemplateParameter[] tmplParams = template.getTemplateParameters();
 		
-		if (tmplArgs != null && !addExplicitArguments(tmplParams, tmplArgs, map, point))
+		if (tmplArgs != null && !addExplicitArguments(template, tmplParams, tmplArgs, map, point))
 			return null;
 				
 		if (!deduceFromFunctionArgs(template, fnArgs, argIsLValue, map, point)) 
@@ -289,6 +292,9 @@ public class TemplateArgumentDeduction {
 						}
 					}
 				}
+				while (pcheck instanceof ITypedef)
+					pcheck = ((ITypedef) pcheck).getType();
+
 				if (pcheck instanceof ICPPTemplateInstance && argcheck instanceof ICPPClassType) {
 					ICPPTemplateInstance pInst = (ICPPTemplateInstance) pcheck;
 					ICPPClassTemplate pTemplate= getPrimaryTemplate(pInst);
@@ -315,7 +321,7 @@ public class TemplateArgumentDeduction {
 	static ICPPTemplateArgument[] deduceForAddressOf(ICPPFunctionTemplate template,
 			ICPPTemplateArgument[] tmplArgs, IFunctionType arg, CPPTemplateParameterMap map, IASTNode point) throws DOMException {
 		final ICPPTemplateParameter[] tmplParams = template.getTemplateParameters();
-		if (!addExplicitArguments(tmplParams, tmplArgs, map, point))
+		if (!addExplicitArguments(template, tmplParams, tmplArgs, map, point))
 			return null;
 				
 		IType par= template.getType();
@@ -369,8 +375,8 @@ public class TemplateArgumentDeduction {
 				final ICPPTemplateParameter tpar = tmplParams[i];
 				ICPPTemplateArgument deducedArg= map.getArgument(tpar);
 				if (deducedArg == null) {
-					deducedArg= tpar.getDefaultValue();
-					if (deducedArg == null)
+					deducedArg= CPPTemplates.instantiateArgument(tpar.getDefaultValue(), map, -1, null, point);
+					if (!CPPTemplates.isValidArgument(deducedArg))
 						return null;
 				}			
 				result[i]= deducedArg;
@@ -387,7 +393,7 @@ public class TemplateArgumentDeduction {
 			ICPPTemplateArgument[] args, ICPPFunctionType ftype, CPPTemplateParameterMap map, IASTNode point) throws DOMException {
 		final ICPPTemplateParameter[] tmplParams = template.getTemplateParameters();
 		
-		if (!addExplicitArguments(tmplParams, args, map, point))
+		if (!addExplicitArguments(template, tmplParams, args, map, point))
 			return null;
 
 		IType a= SemanticUtil.getSimplifiedType(ftype);
@@ -424,6 +430,7 @@ public class TemplateArgumentDeduction {
 			int result= 0;
 			TemplateArgumentDeduction deduct= new TemplateArgumentDeduction(tmplPars, new CPPTemplateParameterMap(0), new CPPTemplateParameterMap(fnParCount), 0);
 			IType fnParPack= null;
+			Set<Integer> usedTemplateParIds= new HashSet<Integer>();
 			for (int j= 0; j < fnArgCount; j++) {
 				IType par;
 				if (fnParPack != null) {
@@ -444,18 +451,94 @@ public class TemplateArgumentDeduction {
 				} 
 				
 				IType arg = fnArgs[j];
+				addReferencedTemplateParameters(par, usedTemplateParIds);
 				int cmpSpecialized= deduceForPartialOrdering(par, arg, deduct, point);
 				if (cmpSpecialized < 0)
 					return cmpSpecialized;
 				if (cmpSpecialized > 0)
 					result= cmpSpecialized;
 			}
-			return result;
+			return verifyDeductionForPartialOrdering(tmplPars, usedTemplateParIds, deduct.fDeducedArgs) ? result : -1;
 		} catch (DOMException e) {
 		}
 		return -1;
 	}
+
+	private static void addReferencedTemplateParameters(IType t, Set<Integer> usedTemplateParIds) {
+		while (true) {
+			if (t instanceof ICPPTemplateParameter) {
+				usedTemplateParIds.add(((ICPPTemplateParameter) t).getParameterID());
+				return;
+			} else if (t instanceof ICPPFunctionType) {
+				final ICPPFunctionType ft = (ICPPFunctionType) t;
+				for (IType par : ft.getParameterTypes())
+					addReferencedTemplateParameters(par, usedTemplateParIds);
+				t = ft.getReturnType();
+			} else if (t instanceof ICPPPointerToMemberType) {
+				ICPPPointerToMemberType ptmt = (ICPPPointerToMemberType) t;
+				addReferencedTemplateParameters(ptmt.getMemberOfClass(), usedTemplateParIds);
+				t = ptmt.getType();
+			} else if (t instanceof ICPPTemplateInstance) {
+				ICPPTemplateInstance inst = (ICPPTemplateInstance) t;
+				for (ICPPTemplateArgument arg : inst.getTemplateArguments()) {
+					if (arg instanceof CPPTemplateTypeArgument)
+						addReferencedTemplateParameters(arg.getTypeValue(), usedTemplateParIds);
+					else
+						addReferencedTemplateParameters(arg.getNonTypeValue(), usedTemplateParIds);
+				}
+				if (inst.getTemplateDefinition() instanceof IType) {
+					t = (IType) inst.getTemplateDefinition();
+				} else {
+					return;
+				}
+			} else if (t instanceof ICPPUnknownMember) {
+				t = ((ICPPUnknownMember) t).getOwnerType();
+			} else if (t instanceof ITypeContainer) {
+				if (t instanceof IArrayType)
+					addReferencedTemplateParameters(((IArrayType) t).getSize(), usedTemplateParIds);
+				t = ((ITypeContainer) t).getType();
+			} else {
+				return;
+			}
+		}
+	}
 	
+	private static void addReferencedTemplateParameters(IValue v, Set<Integer> usedTemplatePars) {
+		if (v != null && v.getEvaluation() instanceof EvalBinding) {
+			IBinding binding = ((EvalBinding) v.getEvaluation()).getBinding();
+			if (binding instanceof ICPPTemplateParameter) {
+				usedTemplatePars.add(((ICPPTemplateParameter) binding).getParameterID());
+			}
+		}
+	}
+
+	/**
+	 *  14.8.2.4-11 [temp.deduction.partial]
+	 *  In most cases, all template parameters must have values in order for deduction to succeed,
+	 *  but for partial ordering purposes a template parameter may remain without a value provided
+	 *  it is not used in the types being used for partial ordering.
+	 */
+	private static boolean verifyDeductionForPartialOrdering(ICPPTemplateParameter[] pars,
+			Set<Integer> usedParIds, CPPTemplateParameterMap tpMap) {
+		for (ICPPTemplateParameter tpar : pars) {
+			if (usedParIds.contains(tpar.getParameterID())) {
+				if (tpar.isParameterPack()) {
+					ICPPTemplateArgument[] deducedArgs = tpMap.getPackExpansion(tpar);
+					if (deducedArgs != null) {
+						for (ICPPTemplateArgument arg : deducedArgs) {
+							if (arg == null)
+								return false;
+						}
+					}
+				} else {
+					if (tpMap.getArgument(tpar) == null)
+						return false;
+				}
+			}
+		}
+		return true;
+	}
+
 	private static int deduceForPartialOrdering(IType par, IType arg, TemplateArgumentDeduction deduct, IASTNode point) throws DOMException {
 		par= getNestedType(par, TDEF);
 		arg= getNestedType(arg, TDEF);
@@ -479,7 +562,7 @@ public class TemplateArgumentDeduction {
 	/**
 	 * Adds the explicit arguments to the map.
 	 */
-	public static boolean addExplicitArguments(final ICPPTemplateParameter[] tmplParams,
+	public static boolean addExplicitArguments(ICPPFunctionTemplate template, final ICPPTemplateParameter[] tmplParams,
 			ICPPTemplateArgument[] tmplArgs, CPPTemplateParameterMap map, IASTNode point) {
 		tmplArgs= SemanticUtil.getSimplifiedArguments(tmplArgs);
 		ICPPTemplateParameter tmplParam= null;
@@ -495,7 +578,7 @@ public class TemplateArgumentDeduction {
 				}
 			}
 			ICPPTemplateArgument tmplArg= tmplArgs[i];
-			tmplArg= CPPTemplates.matchTemplateParameterAndArgument(tmplParam, tmplArg, map, point);
+			tmplArg= CPPTemplates.matchTemplateParameterAndArgument(template, tmplParam, tmplArg, map, point);
 			if (tmplArg == null)
 				return false;
 	
@@ -750,6 +833,8 @@ public class TemplateArgumentDeduction {
 			IType argumentTypeBeforeTypedefResolution = a;
 			while (a instanceof ITypedef)
 				a = ((ITypedef) a).getType();
+			while (p instanceof ITypedef)
+				p = ((ITypedef) p).getType();
 			if (p instanceof IBasicType) {
 				return p.isSameType(a);
 			} else if (p instanceof ICPPPointerToMemberType) {
@@ -765,14 +850,20 @@ public class TemplateArgumentDeduction {
 				p = ptrP.getType();
 				a = ptrA.getType();
 			} else if (p instanceof IPointerType) {
-				if (!(a instanceof IPointerType)) 
-					return false;
 				final IPointerType ptrP = (IPointerType) p;
-				final IPointerType ptrA = (IPointerType) a;
-				if (!allowCVQConversion && (ptrP.isConst() != ptrA.isConst() || ptrP.isVolatile() != ptrA.isVolatile()))
-					return false;
 				p = ptrP.getType();
-				a = ptrA.getType();
+				if (a instanceof TypeOfDependentExpression) {
+					ICPPEvaluation eval = ((TypeOfDependentExpression) a).getEvaluation();
+					eval = new EvalUnary(IASTUnaryExpression.op_star, eval, null, eval.getTemplateDefinition());
+					a = new TypeOfDependentExpression(eval);
+				} else {
+					if (!(a instanceof IPointerType)) 
+						return false;
+					final IPointerType ptrA = (IPointerType) a;
+					if (!allowCVQConversion && (ptrP.isConst() != ptrA.isConst() || ptrP.isVolatile() != ptrA.isVolatile()))
+						return false;
+					a = ptrA.getType();
+				}
 			} else if (p instanceof ICPPReferenceType) {
 				if (!(a instanceof ICPPReferenceType)) {
 					return false;
@@ -922,7 +1013,7 @@ public class TemplateArgumentDeduction {
 
 	private boolean fromFunctionType(ICPPFunctionType ftp, ICPPFunctionType fta, IASTNode point)
 			throws DOMException {
-		if (ftp.isConst() != fta.isConst() || ftp.isVolatile() != fta.isVolatile())
+		if (ftp.isConst() != fta.isConst() || ftp.isVolatile() != fta.isVolatile() || ftp.takesVarArgs() != fta.takesVarArgs())
 			return false;
 		
 		if (!fromType(ftp.getReturnType(), fta.getReturnType(), false, point)) 
