@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2007, 2010 Symbian Software Systems and others.
+ * Copyright (c) 2007, 2013 Symbian Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -24,6 +24,7 @@ import org.eclipse.cdt.core.dom.ast.IType;
 import org.eclipse.cdt.core.dom.ast.ITypedef;
 import org.eclipse.cdt.core.dom.ast.IValue;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPAliasTemplate;
+import org.eclipse.cdt.core.dom.ast.cpp.ICPPAliasTemplateInstance;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPBinding;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPClassScope;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPClassTemplate;
@@ -52,12 +53,14 @@ import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateNonTypeParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateTemplateParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateTypeParameter;
+import org.eclipse.cdt.core.dom.ast.cpp.ICPPUnaryTypeTransformation;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPUsingDeclaration;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPVariable;
 import org.eclipse.cdt.core.index.IIndex;
 import org.eclipse.cdt.core.index.IIndexBinding;
 import org.eclipse.cdt.core.index.IIndexMacroContainer;
 import org.eclipse.cdt.internal.core.dom.parser.Value;
+import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPAliasTemplateInstance;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPArrayType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPFunctionType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPParameterPackType;
@@ -65,6 +68,7 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPPointerToMemberType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPPointerType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPQualifierType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPReferenceType;
+import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPUnaryTypeTransformation;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPClassSpecializationScope;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPDeferredClassInstance;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ICPPEvaluation;
@@ -85,6 +89,7 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalFunctionSet;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalID;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalInitList;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalMemberAccess;
+import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalParameterPack;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalTypeId;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalUnary;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalUnaryTypeID;
@@ -104,9 +109,6 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 		super(index);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.internal.core.index.composite.cpp.ICompositesFactory#getCompositeScope(org.eclipse.cdt.core.index.IIndex, org.eclipse.cdt.core.dom.ast.IScope)
-	 */
 	@Override
 	public IIndexScope getCompositeScope(IIndexScope rscope) {
 		try {
@@ -123,7 +125,7 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			if (rscope instanceof ICPPNamespaceScope) {
 				ICPPNamespace[] namespaces;
 				if (rscope instanceof CompositeCPPNamespace) {
-					// avoid duplicating the search
+					// Avoid duplicating the search.
 					namespaces = ((CompositeCPPNamespace) rscope).namespaces;
 				} else {
 					namespaces = getNamespaces(rscope.getScopeBinding());
@@ -137,9 +139,6 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 		} 
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.internal.core.index.composite.cpp.ICompositesFactory#getCompositeType(org.eclipse.cdt.core.index.IIndex, org.eclipse.cdt.core.dom.ast.IType)
-	 */
 	@Override
 	public IType getCompositeType(IType rtype) {
 		if (rtype instanceof IIndexFragmentBinding) {
@@ -214,13 +213,26 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			}
 			return at;
 		}
+		if (rtype instanceof ICPPAliasTemplateInstance) {
+			ICPPAliasTemplateInstance instance = (ICPPAliasTemplateInstance) rtype;
+			ICPPAliasTemplate aliasTemplate = instance.getTemplateDefinition();
+			if (aliasTemplate instanceof IIndexFragmentBinding)
+				aliasTemplate = (ICPPAliasTemplate) getCompositeBinding((IIndexFragmentBinding) aliasTemplate);
+			IType aliasedType = getCompositeType(instance.getType());
+			return new CPPAliasTemplateInstance(instance.getNameCharArray(), aliasTemplate, aliasedType);
+		}
 		if (rtype instanceof TypeOfDependentExpression) {
-			TypeOfDependentExpression tde= (TypeOfDependentExpression) rtype;
-			ICPPEvaluation e= tde.getEvaluation();
+			TypeOfDependentExpression type= (TypeOfDependentExpression) rtype;
+			ICPPEvaluation e= type.getEvaluation();
 			ICPPEvaluation e2= getCompositeEvaluation(e);
 			if (e != e2)
 				return new TypeOfDependentExpression(e2);
-			return tde;
+			return type;
+		}
+		if (rtype instanceof ICPPUnaryTypeTransformation) {
+			ICPPUnaryTypeTransformation typeTransformation= (ICPPUnaryTypeTransformation) rtype;
+			IType operand = getCompositeType(typeTransformation.getOperand());
+			return new CPPUnaryTypeTransformation(typeTransformation.getOperator(), operand);
 		}
 		if (rtype instanceof IBasicType || rtype == null || rtype instanceof ISemanticProblem) {
 			return rtype;
@@ -232,6 +244,8 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 	public ICPPEvaluation getCompositeEvaluation(ICPPEvaluation eval) {
 		if (eval == null)
 			return null;
+		IBinding templateDefinition = eval.getTemplateDefinition();
+		IBinding templateDefinition2 = getCompositeBinding((IIndexFragmentBinding) templateDefinition);
 		if (eval instanceof EvalBinary) {
 			EvalBinary e= (EvalBinary) eval;
 			ICPPEvaluation a = e.getArg1();
@@ -239,8 +253,8 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			
 			ICPPEvaluation a2 = getCompositeEvaluation(a);
 			ICPPEvaluation b2 = getCompositeEvaluation(b);
-			if (a != a2 || b != b2)
-				e= new EvalBinary(e.getOperator(), a2, b2);
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+				e= new EvalBinary(e.getOperator(), a2, b2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalBinaryTypeId) {
@@ -250,8 +264,8 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			
 			IType a2 = getCompositeType(a);
 			IType b2 = getCompositeType(b);
-			if (a != a2 || b != b2)
-				e= new EvalBinaryTypeId(e.getOperator(), a2, b2);
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+				e= new EvalBinaryTypeId(e.getOperator(), a2, b2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalBinding) {
@@ -261,9 +275,9 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 				IType b = e.getFixedType();
 				IBinding a2 = getCompositeBinding((IIndexFragmentBinding) parameterOwner);
 				IType b2 = getCompositeType(b);
-				if (parameterOwner != a2 || b != b2) {
+				if (parameterOwner != a2 || b != b2 || templateDefinition != templateDefinition2) {
 					int parameterPosition = e.getFunctionParameterPosition();
-					e= new EvalBinding((ICPPFunction) a2, parameterPosition, b2);
+					e= new EvalBinding((ICPPFunction) a2, parameterPosition, b2, templateDefinition2);
 				}
 			} else {
 				IBinding a = e.getBinding();
@@ -271,25 +285,27 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 				
 				IBinding a2 = getCompositeBinding((IIndexFragmentBinding) a);
 				IType b2 = getCompositeType(b);
-				if (a != a2 || b != b2)
-					e= new EvalBinding(a2, b2);
+				if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+					e= new EvalBinding(a2, b2, templateDefinition2);
 			}
 			return e;
 		}
 		if (eval instanceof EvalComma) {
 			EvalComma e= (EvalComma) eval;
 			ICPPEvaluation[] a = e.getArguments();
+			
 			ICPPEvaluation[] a2 = getCompositeEvaluationArray(a);
-			if (a != a2)
-				e= new EvalComma(a2);
+			
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e= new EvalComma(a2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalCompound) {
 			EvalCompound e= (EvalCompound) eval;
 			ICPPEvaluation a = e.getLastEvaluation();
 			ICPPEvaluation a2 = getCompositeEvaluation(a);
-			if (a != a2)
-				e= new EvalCompound(a2);
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e= new EvalCompound(a2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalConditional) {
@@ -300,8 +316,8 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			ICPPEvaluation a2 = getCompositeEvaluation(a);
 			ICPPEvaluation b2 = getCompositeEvaluation(b);
 			ICPPEvaluation c2 = getCompositeEvaluation(c);
-			if (a != a2 || b != b2 || c != c2)
-				e= new EvalConditional(a2, b2, c2, e.isPositiveThrows(), e.isNegativeThrows());
+			if (a != a2 || b != b2 || c != c2 || templateDefinition != templateDefinition2)
+				e= new EvalConditional(a2, b2, c2, e.isPositiveThrows(), e.isNegativeThrows(), templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalFixed) {
@@ -310,7 +326,7 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			IValue b = e.getValue();
 			IType a2 = getCompositeType(a);
 			IValue b2= getCompositeValue(b);
-			if (a != a2 || b != b2)
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
 				e= new EvalFixed(a2, e.getValueCategory(), b2);
 			return e;
 		}
@@ -318,20 +334,25 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			EvalFunctionCall e= (EvalFunctionCall) eval;
 			ICPPEvaluation[] a = e.getArguments();
 			ICPPEvaluation[] a2 = getCompositeEvaluationArray(a);
-			if (a != a2)
-				e= new EvalFunctionCall(a2);
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e= new EvalFunctionCall(a2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalFunctionSet) {
 			EvalFunctionSet e= (EvalFunctionSet) eval;
 			final CPPFunctionSet fset = e.getFunctionSet();
-			ICPPFunction[] a = fset.getBindings();
-			ICPPTemplateArgument[] b = fset.getTemplateArguments();
-			
-			ICPPFunction[] a2 = getCompositeFunctionArray(a);
-			ICPPTemplateArgument[] b2 = TemplateInstanceUtil.convert(this, b);
-			if (a != a2 || b != b2)
-				e= new EvalFunctionSet(new CPPFunctionSet(a2, b2, null), e.isAddressOf());
+			if (fset != null) {
+				ICPPFunction[] a = fset.getBindings();
+				ICPPTemplateArgument[] b = fset.getTemplateArguments();
+				IType c = e.getImpliedObjectType();
+				
+				ICPPFunction[] a2 = getCompositeFunctionArray(a);
+				ICPPTemplateArgument[] b2 = TemplateInstanceUtil.convert(this, b);
+				IType c2 = getCompositeType(c);
+				if (a != a2 || b != b2 || c != c2 || templateDefinition != templateDefinition2)
+					e= new EvalFunctionSet(new CPPFunctionSet(a2, b2, null), e.isQualified(), e.isAddressOf(), 
+							c2, templateDefinition2);
+			}
 			return e;
 		}
 		if (eval instanceof EvalID) {
@@ -349,16 +370,16 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			}
 			ICPPTemplateArgument[] c2 = TemplateInstanceUtil.convert(this, c);
 			
-			if (a != a2 || b != b2 || c != c2)
-				e= new EvalID(a2, b2, e.getName(), e.isAddressOf(), e.isQualified(), c2);
+			if (a != a2 || b != b2 || c != c2 || templateDefinition != templateDefinition2)
+				e= new EvalID(a2, b2, e.getName(), e.isAddressOf(), e.isQualified(), c2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalInitList) {
 			EvalInitList e= (EvalInitList) eval;
 			ICPPEvaluation[] a = e.getClauses();
 			ICPPEvaluation[] a2 = getCompositeEvaluationArray(a);
-			if (a != a2)
-				e= new EvalInitList(a2);
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e= new EvalInitList(a2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalMemberAccess) {
@@ -367,8 +388,16 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			IBinding b = e.getMember();
 			IType a2= getCompositeType(a);
 			IBinding b2= getCompositeBinding((IIndexFragmentBinding) b);
-			if (a != a2 || b != b2)
-				e= new EvalMemberAccess(a2, e.getOwnerValueCategory(), b2, e.isPointerDeref());
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+				e= new EvalMemberAccess(a2, e.getOwnerValueCategory(), b2, e.isPointerDeref(), templateDefinition2);
+			return e;
+		}
+		if (eval instanceof EvalParameterPack) {
+			EvalParameterPack e = (EvalParameterPack) eval;
+			ICPPEvaluation a = e.getExpansionPattern();
+			ICPPEvaluation a2 = getCompositeEvaluation(a);
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e = new EvalParameterPack(a2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalTypeId) {
@@ -377,8 +406,8 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			ICPPEvaluation[] b = e.getArguments();
 			IType a2= getCompositeType(a);
 			ICPPEvaluation[] b2 = getCompositeEvaluationArray(b);
-			if (a != a2 || b != b2)
-				e= new EvalTypeId(a2, b2);
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+				e= new EvalTypeId(a2, templateDefinition2, b2);
 			return e;
 		}
 		if (eval instanceof EvalUnary) {
@@ -387,16 +416,16 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			ICPPEvaluation a2 = getCompositeEvaluation(a);
 			IBinding b= e.getAddressOfQualifiedNameBinding();
 			IBinding b2= getCompositeBinding((IIndexFragmentBinding) b);
-			if (a != a2 || b != b2)
-				e= new EvalUnary(e.getOperator(), a2, b2);
+			if (a != a2 || b != b2 || templateDefinition != templateDefinition2)
+				e= new EvalUnary(e.getOperator(), a2, b2, templateDefinition2);
 			return e;
 		}
 		if (eval instanceof EvalUnaryTypeID) {
 			EvalUnaryTypeID e= (EvalUnaryTypeID) eval;
 			IType a = e.getArgument();
 			IType a2 = getCompositeType(a);
-			if (a != a2)
-				e= new EvalUnaryTypeID(e.getOperator(), a2);
+			if (a != a2 || templateDefinition != templateDefinition2)
+				e= new EvalUnaryTypeID(e.getOperator(), a2, templateDefinition2);
 			return e;
 		}
 		
@@ -465,9 +494,6 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 		return findOneBinding(binding, false);
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.internal.core.index.composite.cpp.ICompositesFactory#getCompositeBinding(org.eclipse.cdt.core.index.IIndex, org.eclipse.cdt.core.dom.ast.IBinding)
-	 */
 	@Override
 	public IIndexBinding getCompositeBinding(IIndexFragmentBinding binding) {
 		IIndexBinding result;
@@ -500,9 +526,7 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 					}
 				} else if (binding instanceof ICPPTemplateDefinition) {
 					if (binding instanceof ICPPClassTemplatePartialSpecialization) {
-						if (binding instanceof ICPPClassTemplatePartialSpecializationSpecialization)
-							return new CompositeCPPClassTemplatePartialSpecializationSpecialization(this, (ICPPClassTemplatePartialSpecializationSpecialization) binding);
-						return new CompositeCPPClassTemplatePartialSpecialization(this, (ICPPClassTemplatePartialSpecialization) findOneBinding(binding));
+						return new CompositeCPPClassTemplatePartialSpecializationSpecialization(this, (ICPPClassTemplatePartialSpecializationSpecialization) binding);
 					} else if (binding instanceof ICPPClassType) {
 						return new CompositeCPPClassTemplateSpecialization(this, (ICPPClassType) binding);
 					} else if (binding instanceof ICPPConstructor) {
@@ -531,10 +555,16 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 						return new CompositeCPPTypedefSpecialization(this, (ICPPBinding) binding);
 					} else if (binding instanceof ICPPUsingDeclaration) {
 						return new CompositeCPPUsingDeclarationSpecialization(this, (ICPPUsingDeclaration) binding);
+					} else if (binding instanceof ICPPEnumeration) {
+						return new CompositeCPPEnumerationSpecialization(this, (ICPPEnumeration) binding);
+					} else if (binding instanceof IEnumerator) {
+						return new CompositeCPPEnumeratorSpecialization(this, (IEnumerator) binding);
 					} else {
 						throw new CompositingNotImplementedError("Composite binding unavailable for " + binding + " " + binding.getClass()); //$NON-NLS-1$ //$NON-NLS-2$
 					}
 				}
+			} else if (binding instanceof ICPPClassTemplatePartialSpecialization) {
+				return new CompositeCPPClassTemplatePartialSpecialization(this, (ICPPClassTemplatePartialSpecialization) findOneBinding(binding));
 			} else if (binding instanceof ICPPTemplateParameter) {
 				if (binding instanceof ICPPTemplateTypeParameter) {
 					result = new CompositeCPPTemplateTypeParameter(this, (ICPPTemplateTypeParameter) binding);
@@ -625,11 +655,13 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 		final long i;
 		final int j;
 		final long k;
+
 		public Key(long id1, int id2, long id3) {
 			i= id1;
 			j= id2;
 			k= id3;
 		}
+
 		@Override
 		public int hashCode() {
 			final int prime = 31;
@@ -639,6 +671,7 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 			result = prime * result + (int) k;
 			return result;
 		}
+
 		@Override
 		public boolean equals(Object obj) {
 			if (obj instanceof Key) {
@@ -652,6 +685,7 @@ public class CPPCompositesFactory extends AbstractCompositeFactory {
 	public static Object createInstanceCacheKey(ICompositesFactory cf, IIndexFragmentBinding rbinding) {
 		return new Key(Thread.currentThread().getId(), cf.hashCode(), rbinding.getBindingID());
 	}
+
 	public static Object createSpecializationKey(ICompositesFactory cf,IIndexFragmentBinding rbinding) {
 		return new Key(Thread.currentThread().getId(), cf.hashCode(), rbinding.getBindingID()+1);
 	}
