@@ -15,6 +15,7 @@ package org.eclipse.cdt.internal.core.parser.scanner;
 import org.eclipse.cdt.core.parser.IGCCToken;
 import org.eclipse.cdt.core.parser.IProblem;
 import org.eclipse.cdt.core.parser.IToken;
+import org.eclipse.cdt.core.parser.IncludeExportPatterns;
 import org.eclipse.cdt.core.parser.OffsetLimitReachedException;
 import org.eclipse.cdt.core.parser.util.CharArrayUtils;
 
@@ -55,6 +56,8 @@ final public class Lexer implements ITokenSequence {
 		public boolean fCreateImageLocations= true;
 		public boolean fSupportSlashPercentComments= false;
 		public boolean fSupportUTFLiterals= true;
+		public boolean fSupportRawStringLiterals= false;
+		public IncludeExportPatterns fIncludeExportPatterns;
 		
 		@Override
 		public Object clone() {
@@ -74,7 +77,7 @@ final public class Lexer implements ITokenSequence {
 	
 	// the input to the lexer
 	private final AbstractCharArray fInput;
-	private int fStart;
+	private final int fStart;
 	private int fLimit;
 
 	// after phase 3 (newline, trigraph, line-splice)
@@ -268,12 +271,14 @@ final public class Lexer implements ITokenSequence {
 			case 'L':
 				switch (d) {
 				case 'R':
-					markPhase3();
-					if (nextCharPhase3() == '"') {
-						nextCharPhase3();
-						return rawStringLiteral(start, 3, IToken.tLSTRING);
+					if (fOptions.fSupportRawStringLiterals) {
+						markPhase3();
+						if (nextCharPhase3() == '"') {
+							nextCharPhase3();
+							return rawStringLiteral(start, 3, IToken.tLSTRING);
+						}
+						restorePhase3();
 					}
-					restorePhase3();
 					break;
 				case '"':
 					nextCharPhase3();
@@ -289,12 +294,14 @@ final public class Lexer implements ITokenSequence {
 				if (fOptions.fSupportUTFLiterals) {
 					switch (d) {
 					case 'R':
-						markPhase3();
-						if (nextCharPhase3() == '"') {
-							nextCharPhase3();
-							return rawStringLiteral(start, 3, c == 'u' ? IToken.tUTF16STRING : IToken.tUTF32STRING);
+						if (fOptions.fSupportRawStringLiterals) {
+							markPhase3();
+							if (nextCharPhase3() == '"') {
+								nextCharPhase3();
+								return rawStringLiteral(start, 3, c == 'u' ? IToken.tUTF16STRING : IToken.tUTF32STRING);
+							}
+							restorePhase3();
 						}
-						restorePhase3();
 						break;
 					case '"':
 						nextCharPhase3();
@@ -307,7 +314,7 @@ final public class Lexer implements ITokenSequence {
 							markPhase3();
 							switch (nextCharPhase3()) {
 							case 'R':
-								if (nextCharPhase3() == '"') {
+								if (fOptions.fSupportRawStringLiterals && nextCharPhase3() == '"') {
 									nextCharPhase3();
 									return rawStringLiteral(start, 4, IToken.tSTRING);
 								}
@@ -324,7 +331,7 @@ final public class Lexer implements ITokenSequence {
 				return identifier(start, 1);
 				
 			case 'R':
-				if (d == '"') {
+				if (fOptions.fSupportRawStringLiterals && d == '"') {
 					nextCharPhase3();
 					return rawStringLiteral(start, 2, IToken.tSTRING);
 				}
@@ -689,21 +696,21 @@ final public class Lexer implements ITokenSequence {
 	}
 
 	private void blockComment(final int start, final char trigger) {
-		// we can ignore line-splices, trigraphs and windows newlines when searching for the '*'
+		// We can ignore line-splices, trigraphs and windows newlines when searching for the '*'
 		int pos= fEndOffset;
 		while (isValidOffset(pos)) {
 			if (fInput.get(pos++) == trigger) {
 				fEndOffset= pos;
 				if (nextCharPhase3() == '/') {
 					nextCharPhase3();
-					fLog.handleComment(true, start, fOffset);
+					fLog.handleComment(true, start, fOffset, fInput);
 					return;
 				}
 			}
 		}
 		fCharPhase3= END_OF_INPUT;
 		fOffset= fEndOffset= pos;
-		fLog.handleComment(true, start, pos);
+		fLog.handleComment(true, start, pos, fInput);
 	}
 
 	private void lineComment(final int start) {
@@ -712,7 +719,7 @@ final public class Lexer implements ITokenSequence {
 			switch (c) {
 			case END_OF_INPUT:
 			case '\n':
-				fLog.handleComment(false, start, fOffset);
+				fLog.handleComment(false, start, fOffset, fInput);
 				return;
 			}
 			c= nextCharPhase3();
