@@ -14,9 +14,15 @@
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.pdom.dom.cpp;
 
+import java.util.ArrayList;
+import java.util.LinkedList;
+import java.util.List;
+
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.dom.ast.DOMException;
 import org.eclipse.cdt.core.dom.ast.IASTCompositeTypeSpecifier;
+import org.eclipse.cdt.core.dom.ast.IASTDeclSpecifier;
+import org.eclipse.cdt.core.dom.ast.IASTFunctionDefinition;
 import org.eclipse.cdt.core.dom.ast.IASTName;
 import org.eclipse.cdt.core.dom.ast.IASTNode;
 import org.eclipse.cdt.core.dom.ast.IASTSimpleDeclaration;
@@ -89,6 +95,7 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPPointerToMemberType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPPointerType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPQualifierType;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPReferenceType;
+import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPUnaryTypeTransformation;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPUnknownClassInstance;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.CPPUnknownMember;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.ClassTypeHelper;
@@ -113,6 +120,7 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalTypeId;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalUnary;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.EvalUnaryTypeID;
 import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.TypeOfDependentExpression;
+import org.eclipse.cdt.internal.core.index.IIndexBindingConstants;
 import org.eclipse.cdt.internal.core.index.IIndexCPPBindingConstants;
 import org.eclipse.cdt.internal.core.index.composite.CompositeIndexBinding;
 import org.eclipse.cdt.internal.core.pdom.PDOM;
@@ -130,10 +138,6 @@ import org.eclipse.cdt.internal.core.pdom.dom.PDOMName;
 import org.eclipse.cdt.internal.core.pdom.dom.PDOMNode;
 import org.eclipse.core.runtime.CoreException;
 
-import java.util.ArrayList;
-import java.util.LinkedList;
-import java.util.List;
-
 /**
  * Container for c++-entities.
  */
@@ -176,7 +180,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 
 	@Override
 	public int getNodeType() {
-		return LINKAGE;
+		return IIndexBindingConstants.LINKAGE;
 	}
 
 	// Binding types
@@ -247,6 +251,22 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 					fReturnExpression);
 		}
 	}
+	
+	class ConfigureFunctionSpecialization implements Runnable {
+		private final PDOMCPPFunctionSpecialization fSpec; 
+		private final ICPPEvaluation fReturnExpression;
+		
+		public ConfigureFunctionSpecialization(ICPPFunction original, PDOMCPPFunctionSpecialization spec) {
+			fSpec = spec;
+			fReturnExpression = CPPFunction.getReturnExpression(original);
+			postProcesses.add(this);
+		}
+		
+		@Override
+		public void run() {
+			fSpec.initData(fReturnExpression);
+		}
+	}
 
 	class ConfigureFunctionTemplate implements Runnable {
 		private final PDOMCPPFunctionTemplate fTemplate;
@@ -341,7 +361,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 
 		PDOMBinding pdomBinding= attemptFastAdaptBinding(inputBinding);
 		if (pdomBinding == null) {
-			// assign names to anonymous types.
+			// Assign names to anonymous types.
 			IBinding binding= PDOMASTAdapter.getAdapterForAnonymousASTBinding(inputBinding);
 			if (binding == null)
 				return null;
@@ -518,6 +538,9 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
         if (binding instanceof ICPPClassTemplatePartialSpecialization) {
         	// A class template partial specialization inherits the visibility of its primary class template. 
         	binding = ((ICPPClassTemplatePartialSpecialization) binding).getPrimaryClassTemplate();
+        }
+        if (binding instanceof ICPPAliasTemplateInstance) {
+        	binding = ((ICPPAliasTemplateInstance) binding).getTemplateDefinition();
         }
 		if (binding instanceof CPPImplicitMethod)
 			return ICPPClassType.v_public;
@@ -1036,17 +1059,20 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 				}
 			}
 		} else if (parentNode instanceof ICPPASTFunctionDeclarator) {
+			IASTDeclSpecifier declSpec = null;
 			if (parentNode.getParent() instanceof IASTSimpleDeclaration) {
-				IASTSimpleDeclaration grandparentNode = (IASTSimpleDeclaration) parentNode.getParent();
-				if (grandparentNode.getDeclSpecifier() instanceof ICPPASTDeclSpecifier) {
-					if (((ICPPASTDeclSpecifier) grandparentNode.getDeclSpecifier()).isFriend()) {
-						pdomName.setIsFriendSpecifier();
-						PDOMName enclClassName = (PDOMName) pdomName.getEnclosingDefinition();
-						if (enclClassName != null) {
-							PDOMBinding enclClassBinding = enclClassName.getBinding();
-							if (enclClassBinding instanceof PDOMCPPClassType) {
-								((PDOMCPPClassType) enclClassBinding).addFriend(new PDOMCPPFriend(this,	pdomName));
-							}
+				declSpec = ((IASTSimpleDeclaration) parentNode.getParent()).getDeclSpecifier();
+			} else if (parentNode.getParent() instanceof IASTFunctionDefinition) {
+				declSpec = ((IASTFunctionDefinition) parentNode.getParent()).getDeclSpecifier();
+			}
+			if (declSpec instanceof ICPPASTDeclSpecifier) {
+				if (((ICPPASTDeclSpecifier) declSpec).isFriend()) {
+					pdomName.setIsFriendSpecifier();
+					PDOMName enclClassName = (PDOMName) pdomName.getEnclosingDefinition();
+					if (enclClassName != null) {
+						PDOMBinding enclClassBinding = enclClassName.getBinding();
+						if (enclClassBinding instanceof PDOMCPPClassType) {
+							((PDOMCPPClassType) enclClassBinding).addFriend(new PDOMCPPFriend(this,	pdomName));
 						}
 					}
 				}
@@ -1193,6 +1219,8 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			return CPPDeferredClassInstance.unmarshal(getPDOM(), firstBytes, buffer);
 		case ITypeMarshalBuffer.ALIAS_TEMPLATE:
 			return CPPAliasTemplateInstance.unmarshal(firstBytes, buffer);
+		case ITypeMarshalBuffer.TYPE_TRANSFORMATION:
+			return CPPUnaryTypeTransformation.unmarshal(firstBytes, buffer);
 		}
 
 		throw new CoreException(CCorePlugin.createStatus("Cannot unmarshal a type, first bytes=" + firstBytes)); //$NON-NLS-1$
