@@ -6,8 +6,8 @@
  * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
- *    Markus Schorn - initial API and implementation
- *******************************************************************************/ 
+ *     Markus Schorn - initial API and implementation
+ *******************************************************************************/
 package org.eclipse.cdt.ui.tests.typehierarchy;
 
 import junit.framework.Test;
@@ -18,9 +18,8 @@ import org.eclipse.swt.widgets.TreeItem;
 
 import org.eclipse.cdt.internal.ui.editor.CEditor;
 
-
 public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
-	
+
 	public CppTypeHierarchyTest(String name) {
 		super(name);
 	}
@@ -56,68 +55,65 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("Simple1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
-		
+
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		item4= checkTreeNode(item1, 1, "Simple4");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Simple2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Simple3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Simple4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item4= checkTreeNode(item1, 0, "Simple4");
 		assertEquals(1, item1.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
 	}
@@ -149,68 +145,65 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("field1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
-		
+
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		item4= checkTreeNode(item1, 1, "Simple4");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("field3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "Simple2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "Simple3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Simple1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item4= checkTreeNode(item1, 0, "Simple4");
 		assertEquals(1, item1.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
 	}
@@ -239,11 +232,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "multi.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("Multi1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -251,17 +244,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Multi1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Multi4");
 		assertEquals(1, item3.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Multi2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -272,22 +264,21 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		item3= checkTreeNode(item2, 0, "Multi3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Multi4");
 		assertEquals(1, item3.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Multi3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Multi1");
-		item2= checkTreeNode(tree, 1, "Multi2");		
+		item2= checkTreeNode(tree, 1, "Multi2");
 		assertEquals(2, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
 		item4= checkTreeNode(item3, 0, "Multi4");
@@ -302,15 +293,14 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Multi4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Multi1");
-		item2= checkTreeNode(tree, 1, "Multi2");		
+		item2= checkTreeNode(tree, 1, "Multi2");
 		assertEquals(2, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
 		item4= checkTreeNode(item3, 0, "Multi4");
@@ -350,11 +340,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "multimem.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("field1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -362,17 +352,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Multi1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Multi4");
 		assertEquals(1, item3.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -383,22 +372,21 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		item3= checkTreeNode(item2, 0, "Multi3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Multi4");
 		assertEquals(1, item3.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("field3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Multi1");
-		item2= checkTreeNode(tree, 1, "Multi2");		
+		item2= checkTreeNode(tree, 1, "Multi2");
 		assertEquals(2, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
 		item4= checkTreeNode(item3, 0, "Multi4");
@@ -413,15 +401,14 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "Multi1");
-		item2= checkTreeNode(tree, 1, "Multi2");		
+		item2= checkTreeNode(tree, 1, "Multi2");
 		assertEquals(2, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Multi3");
 		assertEquals(1, item1.getItemCount());
 		item4= checkTreeNode(item3, 0, "Multi4");
@@ -461,11 +448,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "diamond.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("Diamond1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -473,11 +460,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		item3= checkTreeNode(item1, 1, "Diamond3");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
@@ -488,7 +475,6 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Diamond2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -496,17 +482,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Diamond3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -514,17 +499,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Diamond3");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Diamond4");
 		assertEquals(1, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("Diamond4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -532,11 +516,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		item3= checkTreeNode(item1, 1, "Diamond3");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
@@ -546,8 +530,8 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
-	}	
-	
+	}
+
 	// class Diamond1 {
 	// public:
 	//    int field1;
@@ -572,11 +556,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "diamondmem.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("field1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -584,11 +568,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		item3= checkTreeNode(item1, 1, "Diamond3");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
@@ -599,7 +583,6 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -607,17 +590,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("field3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -625,17 +607,16 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item3= checkTreeNode(item1, 0, "Diamond3");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item3, 0, "Diamond4");
 		assertEquals(1, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
@@ -643,11 +624,11 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		item1= checkTreeNode(tree, 0, "Diamond1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "Diamond2");
 		item3= checkTreeNode(item1, 1, "Diamond3");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item4= checkTreeNode(item2, 0, "Diamond4");
 		assertEquals(1, item2.getItemCount());
 		assertEquals(0, item4.getItemCount());
@@ -657,7 +638,7 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		assertEquals(0, item4.getItemCount());
 
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
-	}	
+	}
 
 	// class ViaTypedef1 {
 	// public:
@@ -680,72 +661,69 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "viaTypedef.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("ViaTypedef1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
-		
+
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		item4= checkTreeNode(item1, 1, "ViaTypedef4");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("ViaTypedef2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {});
 
-		
 		editor.selectAndReveal(content.indexOf("ViaTypedef3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("ViaTypedef4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item4= checkTreeNode(item1, 0, "ViaTypedef4");
 		assertEquals(1, item1.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
 	}
@@ -771,72 +749,69 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "viaTypedefmem.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("field1"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
-		
+
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
 		getHierarchyViewer().expandAll();
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		item4= checkTreeNode(item1, 1, "ViaTypedef4");
 		assertEquals(2, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("ViaTypedef2"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {});
 
-		
 		editor.selectAndReveal(content.indexOf("field3"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item2= checkTreeNode(item1, 0, "ViaTypedef2");
 		assertEquals(1, item1.getItemCount());
-		
+
 		item3= checkTreeNode(item2, 0, "ViaTypedef3");
 		assertEquals(1, item2.getItemCount());
-		
+
 		assertEquals(0, item3.getItemCount());
 		checkMethodTable(new String[] {"field3 : int", "method3() : int"});
 
-		
 		editor.selectAndReveal(content.indexOf("method4"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
 		item1= checkTreeNode(tree, 0, "ViaTypedef1");
 		assertEquals(1, tree.getItemCount());
-		
+
 		item4= checkTreeNode(item1, 0, "ViaTypedef4");
 		assertEquals(1, item1.getItemCount());
-		
+
 		assertEquals(0, item4.getItemCount());
 		checkMethodTable(new String[] {"field4 : int", "method4() : int"});
 	}
@@ -850,18 +825,62 @@ public class CppTypeHierarchyTest extends TypeHierarchyBaseTest {
 		String content= getContentsForTest(1)[0].toString();
 		IFile file= createFile(getProject(), "simpleTemplate.cpp", content);
 		waitUntilFileIsIndexed(fIndex, file);
-		
+
 		CEditor editor= openEditor(file);
 		Tree tree;
 		TreeItem item1, item2, item3, item4;
-		
+
 		editor.selectAndReveal(content.indexOf("SimpleTemplate"), 1);
 		openTypeHierarchy(editor);
 		tree= getHierarchyViewer().getTree();
-		
+
 		item1= checkTreeNode(tree, 0, "SimpleTemplate");
 		assertEquals(1, tree.getItemCount());
 		assertEquals(0, item1.getItemCount());
 		checkMethodTable(new String[] {"field1 : T", "method1() : T"});
 	}
+
+	//	template<typename T>
+	//	class A {
+	// 	public:
+	//    int field1;
+	//    int method1();
+	// 	};
+	// 	class B : public A<int> {
+	//	public:
+	//	  int field2;
+	//	  int method2();
+	//	};
+	public void testTemplateInstance_403418() throws Exception {
+		String content= getContentsForTest(1)[0].toString();
+		IFile file= createFile(getProject(), "class.cpp", content);
+		waitUntilFileIsIndexed(fIndex, file);
+		CEditor editor= openEditor(file);
+		Tree tree;
+		TreeItem item1, item2;
+
+		editor.selectAndReveal(content.indexOf("A"), 1);
+		openTypeHierarchy(editor);
+		tree= getHierarchyViewer().getTree();
+
+		item1= checkTreeNode(tree, 0, "A");
+		assertEquals(1, tree.getItemCount());
+		getHierarchyViewer().expandAll();
+
+		item2= checkTreeNode(item1, 0, "B");
+		checkMethodTable(new String[] {"field1 : int", "method1() : int"});
+		assertEquals(1, item1.getItemCount());
+		assertEquals(0, item2.getItemCount());
+
+		editor.selectAndReveal(content.indexOf("B"), 1);
+		openTypeHierarchy(editor);
+		tree= getHierarchyViewer().getTree();
+		item1= checkTreeNode(tree, 0, "A");
+		assertEquals(1, tree.getItemCount());
+
+		item2= checkTreeNode(item1, 0, "B");
+		assertEquals(1, item1.getItemCount());
+		assertEquals(0, item2.getItemCount());
+		checkMethodTable(new String[] {"field2 : int", "method2() : int"});
+	}
 }
