@@ -552,7 +552,17 @@ public final class PreferencesMessages extends NLS {
 	public static String ScalabilityPreferencePage_note;
 	public static String ScalabilityPreferencePage_preferenceOnlyForNewEditors;
 	public static String ScalabilityPreferencePage_contentAssist_autoActivation;
-	
+	public static String ScalabilityPreferencePage_parserSettings_group_label;
+	public static String ScalabilityPreferencePage_skipTrivialExpressions_label;
+	public static String ScalabilityPreferencePage_maximumTrivialExpressions_label;
+
+	public static String IndexerStrategyBlock_activeBuildConfig;
+	public static String IndexerStrategyBlock_autoUpdate;
+	public static String IndexerStrategyBlock_buildConfigGroup;
+	public static String IndexerStrategyBlock_immediateUpdate;
+	public static String IndexerStrategyBlock_specificBuildConfig;
+	public static String IndexerStrategyBlock_strategyGroup;
+
 	private PreferencesMessages() {
 		// Do not instantiate
 	}
