@@ -6,27 +6,20 @@
  * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
- *     Alvaro Sanchez-Leon (Ericsson AB) - Support for Step into selection (bug 244865)
+ *     Marc Khouzam (Ericsson) - Support for Step into selection (bug 244865)
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_6_8;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_2;
 
-import org.eclipse.cdt.dsf.gdb.IGDBLaunchConfigurationConstants;
 import org.eclipse.cdt.tests.dsf.gdb.framework.BackgroundRunner;
 import org.eclipse.cdt.tests.dsf.gdb.tests.ITestConstants;
-import org.eclipse.cdt.tests.dsf.gdb.tests.StepIntoSelectionTest;
+import org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_1.StepIntoSelectionTest_7_1;
 import org.junit.runner.RunWith;
 
+
 @RunWith(BackgroundRunner.class)
-public class StepIntoSelectionTest_6_8 extends StepIntoSelectionTest {
+public class StepIntoSelectionTest_7_2 extends StepIntoSelectionTest_7_1 {
     @Override
 	protected void setGdbVersion() {
-		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_6_8);
-	}
-    
-	@Override
-	protected void setLaunchAttributes() {
-		super.setLaunchAttributes();
-		
-    	setLaunchAttribute(IGDBLaunchConfigurationConstants.ATTR_DEBUGGER_NON_STOP, false);
+		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_2);		
 	}
 }
