@@ -8,7 +8,7 @@
  * Contributors:
  *     Ericsson			  - Initial Implementation
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_1;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_2;
 
 import org.eclipse.cdt.dsf.mi.service.command.commands.Suite_Sessionless_Tests;
 import org.eclipse.cdt.tests.dsf.gdb.framework.BaseTestCase;
@@ -23,37 +23,39 @@ import org.junit.runners.Suite;
  * want to run.  When creating a new test class, it should be
  * added to the list below.
  * 
- * This suite is for tests to be run with GDB 7.1
+ * This suite is for tests to be run with GDB 7.2
  */
 
 @RunWith(Suite.class)
 @Suite.SuiteClasses({
 	// We need specific name for the tests of this suite, because of bug https://bugs.eclipse.org/172256
-	MIRegistersTest_7_1.class,
-	MIRunControlTest_7_1.class,
-	MIRunControlTargetAvailableTest_7_1.class,
-	MIRunControlNonStopTargetAvailableTest_7_1.class,
-	MIExpressionsTest_7_1.class,
-	GDBPatternMatchingExpressionsTest_7_1.class,
-	MIMemoryTest_7_1.class,
-	MIBreakpointsTest_7_1.class,
-	MICatchpointsTest_7_1.class,
-	MIDisassemblyTest_7_1.class,
-	GDBProcessesTest_7_1.class,
-	LaunchConfigurationAndRestartTest_7_1.class,
-	OperationsWhileTargetIsRunningTest_7_1.class,
-	OperationsWhileTargetIsRunningNonStopTest_7_1.class,
-	PostMortemCoreTest_7_1.class,
-	CommandTimeoutTest_7_1.class,
-	GDBMultiNonStopRunControlTest_7_1.class,
+	MIRegistersTest_7_2.class,
+	MIRunControlTest_7_2.class,
+	MIRunControlTargetAvailableTest_7_2.class,
+	MIRunControlNonStopTargetAvailableTest_7_2.class,
+	MIExpressionsTest_7_2.class,
+	GDBPatternMatchingExpressionsTest_7_2.class,
+	MIMemoryTest_7_2.class,
+	MIBreakpointsTest_7_2.class,
+	MICatchpointsTest_7_2.class,
+	MIDisassemblyTest_7_2.class,
+	GDBProcessesTest_7_2.class,
+	LaunchConfigurationAndRestartTest_7_2.class,
+	OperationsWhileTargetIsRunningTest_7_2.class,
+	OperationsWhileTargetIsRunningNonStopTest_7_2.class,
+	PostMortemCoreTest_7_2.class,
+	CommandTimeoutTest_7_2.class,
+	GDBMultiNonStopRunControlTest_7_2.class,
 	Suite_Sessionless_Tests.class,
+	StepIntoSelectionTest_7_2.class,
+	StepIntoSelectionTest_7_2_NS.class,
 	/* Add your test class here */
 })
 
-public class Suite_7_1 {
+public class Suite_7_2 {
 	@BeforeClass
 	public static void beforeClassMethod() {
-		BaseTestCase.setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_1);
+		BaseTestCase.setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_2);
 		BaseTestCase.ignoreIfGDBMissing();
 	}
 }
